// The xtend program that reads a TDL Specification file and generates the ttcn-3 code
// Data Set and Test Configuation TDL elements can be parsed and transformed to TTCN-3 code 
package org.example.domainmodel.generator
 
import org.eclipse.emf.ecore.resource.Resource
import org.eclipse.xtext.generator.IGenerator
import org.eclipse.xtext.generator.IFileSystemAccess
import org.example.domainmodel.domainmodel.DataSet
import org.example.domainmodel.domainmodel.DataInstance
import org.example.domainmodel.domainmodel.TestConfiguration
import org.example.domainmodel.domainmodel.TestDescription
import org.example.domainmodel.domainmodel.ComponentInstance
import org.example.domainmodel.domainmodel.GateType

import org.eclipse.xtext.naming.IQualifiedNameProvider
 
import com.google.inject.Inject

class DomainmodelGenerator implements IGenerator {
 
 
 
  @Inject extension IQualifiedNameProvider
	var String dataSet = ""
    var String gate =""
    var String defaultGate =""
    var String importHdr =""
	
    def String getDataSet(Resource resource)
    {
    	var String values="module DataModule \n{\n"
    	for (e : resource.allContents.toIterable.filter(DataSet)) {
             values += e.compileDataSet
       }
       return values
    }
    override void doGenerate(Resource resource, IFileSystemAccess fsa) {
// Data Set module	
       fsa.generateFile("DataModule.ttcn3",getDataSet(resource)+"}")	 
       importHdr = "   import from DataModule type " 
       for (x : resource.allContents.toIterable.filter(DataSet)) {       	   
           	dataSet += x.fullyQualifiedName.toString() + "Type,";
       }
       dataSet = dataSet.substring(0,dataSet.length()-1);  
       dataSet += ";\n";
// Gate element      
       for (x : resource.allContents.toIterable.filter(GateType)) {       	   
           	gate += x.fullyQualifiedName.toString();
       }
       if (gate != "") {
       	defaultGate = gate
       	gate = "\ttype port " + gate + " message {\n"
       }
         
       dataSet = dataSet.substring(dataSet.indexOf(" ")+1); 
       var int pos =0
       var int commaPos = dataSet.indexOf(",",pos)
       var String portStr = "\t\tinout " + dataSet.substring(pos,commaPos) + ";\n"
       pos = commaPos
       portStr += "\t\tinout " +dataSet.substring(pos+1) + "\t}\n"
       var String componentStr=""
       
// Test Configuration module
       for (q : resource.allContents.toIterable.filter(TestConfiguration)) { 
            componentStr += q.compileTestConfig 	
       }        	        	    
       for (e : resource.allContents.toIterable.filter(TestConfiguration)) {
            fsa.generateFile(
                   e.fullyQualifiedName.toString("/") + ".ttcn3",
                   e.compileTestConfigHdr + importHdr + dataSet + gate+portStr + componentStr)
       }  
// Test Description module
       for (e : resource.allContents.toIterable.filter(TestDescription)) {
            fsa.generateFile(
                   e.fullyQualifiedName.toString("/") + ".ttcn3",
                   e.compileTestDescHdr )//+ importHdr + dataSet + gate+portStr + componentStr)
       }  
       
           
    }
//  Template expression 
       def compileTestDescHdr (TestDescription e) '''
       module «e.name»
       {
       	 import from «e.testConfiguration.name» all;
       
       }
       '''
 
    
       def compileTestConfigHdr (TestConfiguration e) '''
       module «e.name»
       {
       '''
       def compileTestConfig (TestConfiguration e) '''
       	  «FOR t : e.componentInstance»
           «t.compileComponent»
          «ENDFOR»
       }
       '''
       def compileComponent (ComponentInstance e) '''
           «"\t"»type component «e.name» {
           «"\t\t"»port «defaultGate» «"g"»«e.name» «";\n\t}"»
       '''   
       def compileDataSet (DataSet e) '''
 		«var String dataset = e.name »
 		«"\t"»type record «dataset»Type {
       	  «FOR d : e.dataInstance»
           «d.compileRecord»
          «ENDFOR» 
 		«"\t"»} 
 		«"\t"»template «e.name»Type TemplateType := {
       	  «FOR t : e.dataInstance»
                  «t.compileTemplate»
          «ENDFOR»  
 		«"\t"»}
       ''' 
      def compileRecord (DataInstance d) '''
          «"\t\t"» Temptype «d.name»
       '''
      def compileTemplate (DataInstance t) '''
          «"\t\t"» «t.name» := "TempValue"  
       '''        
}
